import java.util.*;

import com.evelopers.common.exception.*;
import com.evelopers.unimod.core.stateworks.*;
import com.evelopers.unimod.runtime.*;
import com.evelopers.unimod.runtime.context.*;

/**
 * ,   XML-.
 */
public class Model1EventProcessor extends AbstractEventProcessor {
    //    ,    
    private ModelStructure modelStructure;

    //   A1  A2
    private static final int A1 = 1;
    private static final int A2 = 2;

    //  ,     
    //   
    private int decodeStateMachine(String sm) {
        if ("A1".equals(sm)) {
            return A1;
        } else if ("A2".equals(sm)) {
            return A2;
        }
        return -1;
    }

    //    A1  A2
    private A1EventProcessor _A1;
    private A2EventProcessor _A2;

    //   
    public Model1EventProcessor() {
        modelStructure = new Model1ModelStructure();
        _A1 = new A1EventProcessor();
        _A2 = new A2EventProcessor();
    }

    //     ,
    //    
    public ModelStructure getModelStructure() {
        return modelStructure;
    }

    //    
    public void setControlledObjectsMap(
        ControlledObjectsMap controlledObjectsMap) {
        super.setControlledObjectsMap(controlledObjectsMap);
        _A1.init(controlledObjectsMap);
        _A2.init(controlledObjectsMap);
    }

    //    ,    
    //      
    protected StateMachineConfig process(Event event,
        StateMachineContext context, StateMachinePath path,
        StateMachineConfig config) throws SystemException {

        // get state machine from path
        int sm = decodeStateMachine(path.getStateMachine());
        try {
            switch (sm) {
            case A1:
                return _A1.process(event, context, path, config);
            case A2:
                return _A2.process(event, context, path, config);
            default:
                throw new EventProcessorException(
                    "Unknown state machine ["
                        + path.getStateMachine() + "]");
            }
        } catch (Exception e) {
            if (e instanceof SystemException) {
                throw (SystemException) e;
            } else {
                throw new SystemException(e);
            }
        }
    }

    //   
    protected StateMachineConfig transiteToStableState(
        StateMachineContext context, StateMachinePath path,
        StateMachineConfig config) throws SystemException {

        // get state machine from path
        int sm = decodeStateMachine(path.getStateMachine());
        try {
            switch (sm) {
            case A1:
                return _A1.transiteToStableState(context, path,
                    config);
            case A2:
                return _A2.transiteToStableState(context, path,
                    config);
            default:
                throw new EventProcessorException(
                    "Unknown state machine ["
                        + path.getStateMachine() + "]");
            }
        } catch (Exception e) {
            if (e instanceof SystemException) {
                throw (SystemException) e;
            } else {
                throw new SystemException(e);
            }
        }
    }

    //  ,    
    private class Model1ModelStructure implements ModelStructure {

        private Map configManagers = new HashMap();

        private Model1ModelStructure() {
            configManagers.put("A1",
                new com.evelopers.unimod.runtime.config
                    .DistinguishConfigManager());
            configManagers.put("A2",
                new com.evelopers.unimod.runtime.config
                    .DistinguishConfigManager());
        }

        public StateMachinePath getRootPath()
            throws EventProcessorException {
            return new StateMachinePath("A1");
        }

        public StateMachineConfigManager getConfigManager(
            String stateMachine) throws EventProcessorException {
            return (StateMachineConfigManager) configManagers.get(
                stateMachine);
        }

        public StateMachineConfig getTopConfig(String stateMachine)
            throws EventProcessorException {
            int sm = decodeStateMachine(stateMachine);
            switch (sm) {
            case A1:
                return new StateMachineConfig("Top");
            case A2:
                return new StateMachineConfig("Top");
            default:
                throw new EventProcessorException(
                    "Unknown state machine [" + stateMachine + "]");
            }
        }

        // ,     
        public boolean isFinal(String stateMachine,
            StateMachineConfig config) throws EventProcessorException {
            // get state machine from path
            int sm = decodeStateMachine(stateMachine);
            int state;
            switch (sm) {
            case A1:
                state = _A1.decodeState(config.getActiveState());
                switch (state) {
                case A1EventProcessor._2___:
                    return true;
                default:
                    return false;
                }
            case A2:
                state = _A2.decodeState(config.getActiveState());
                switch (state) {
                case A2EventProcessor._6___:
                    return true;
                default:
                    return false;
                }
            default:
                throw new EventProcessorException(
                    "Unknown state machine [" + stateMachine + "]");
            }
        }
    }

    /**
     * ,   A1.
     */
    private class A1EventProcessor {
        //    (  Top 
        //   )
        private static final int Top = 1;
        private static final int _2___ = 2;
        private static final int _0___ = 3;
        private static final int _1____ = 4;

        //  ,     
        //   
        private int decodeState(String state) {
            if ("Top".equals(state)) {
                return Top;
            } else if ("2.  ".equals(state)) {
                return _2___;
            } else if ("0.  ".equals(state)) {
                return _0___;
            } else if ("1.   ".equals(state)) {
                return _1____;
            }
            return -1;
        }

        //  
        private static final int e1 = 1;
        private static final int e3 = 2;
        private static final int e6 = 3;

        //  ,     
        //   
        private int decodeEvent(String event) {
            if ("e1".equals(event)) {
                return e1;
            } else if ("e3".equals(event)) {
                return e3;
            } else if ("e6".equals(event)) {
                return e6;
            }
            return -1;
        }

        //    A1
        private co.CoinMechanism o2;
        private co.Display o1;

        //     A1
        private void init(ControlledObjectsMap controlledObjectsMap) {
            o2 = (co.CoinMechanism)
                controlledObjectsMap.getControlledObject("o2");
            o1 = (co.Display)
                controlledObjectsMap.getControlledObject("o1");
        }

        //    :   
        // ,    
        private StateMachineConfig process(Event event,
            StateMachineContext context, StateMachinePath path,
            StateMachineConfig config) throws Exception {
            config = lookForTransition(event, context, path, config);
            config = transiteToStableState(context, path, config);

            //   
            executeSubmachines(event, context, path, config);
            return config;
        }

        //    
        private void executeSubmachines(Event event,
            StateMachineContext context, StateMachinePath path,
            StateMachineConfig config) throws Exception {
            int state = decodeState(config.getActiveState());
            while (true) {
                switch (state) {
                case _2___:
                    return;
                case _0___:
                    return;
                case _1____:
                    // 1.    includes A2
                    fireBeforeSubmachineExecution(context, event, path,
                        "1.   ", "A2");
                    Model1EventProcessor.this.process(event, context,
                        new StateMachinePath(path,
                            "1.   ", "A2"));
                    fireAfterSubmachineExecution(context, event, path,
                        "1.   ", "A2");
                    return;
                default:
                    throw new EventProcessorException(
                        "State with name [" + config.getActiveState()
                            + "] is unknown for state machine [A1]");
                }
            }
        }

        //    A1
        private StateMachineConfig transiteToStableState(
            StateMachineContext context, StateMachinePath path,
            StateMachineConfig config) throws Exception {
            int s = decodeState(config.getActiveState());
            Event event;
            switch (s) {
            case Top:
                fireComeToState(context, path,
                    "0.  ");

                // 0.  ->
                // 1.    [true]/o1.z1,o1.z11
                event = Event.NO_EVENT;
                fireTransitionFound(context, path,
                    "0.  ", event,
                    "0.  #1.   ##true");
                fireBeforeOutputActionExecution(context, path,
                    "0.  #1.   ##true",
                    "o1.z1");
                o1.z1(context);
                fireAfterOutputActionExecution(context, path,
                    "0.  #1.   ##true",
                    "o1.z1");
                fireBeforeOutputActionExecution(context, path,
                    "0.  #1.   ##true",
                    "o1.z11");
                o1.z11(context);
                fireAfterOutputActionExecution(context, path,
                    "0.  #1.   ##true",
                    "o1.z11");
                fireComeToState(context, path,
                    "1.   ");

                // 1.    []
                return new StateMachineConfig(
                    "1.   ");
            }
            return config;
        }

        //    (   
        //   ).   ,
        //   ,
        //    .
        private StateMachineConfig lookForTransition(Event event,
            StateMachineContext context, StateMachinePath path,
            StateMachineConfig config) throws Exception {
            boolean o2_x3 = false,
                o2_x1 = false,
                o2_x4 = false;
            BitSet calculatedInputActions = new BitSet(3);
            int s = decodeState(config.getActiveState());
            int e = decodeEvent(event.getName());
            while (true) {
                switch (s) {
                case _1____:
                    switch (e) {
                    case e1:
                        // 1.   ->
                        // 1.    e1[!o2.x1]/o2.z2,o1.z2
                        fireTransitionCandidate(context, path,
                            "1.   ", event,
                            "1.   #" +
                                "1.   #e1#!o2.x1");
                        if (!isInputActionCalculated(
                            calculatedInputActions, _o2_x1)) {
                            fireBeforeInputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e1#!o2.x1",
                                "o2.x1");
                            o2_x1 = o2.x1(context);
                            fireAfterInputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e1#!o2.x1",
                                "o2.x1", new Boolean(o2_x1));
                        }
                        if (!o2_x1) {
                            fireTransitionFound(context, path,
                                "1.   ", event,
                                "1.   #" +
                                    "1.   #e1#!o2.x1");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e1#!o2.x1",
                                "o2.z2");
                            o2.z2(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e1#!o2.x1",
                                "o2.z2");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e1#!o2.x1",
                                "o1.z2");
                            o1.z2(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e1#!o2.x1",
                                "o1.z2");
                            fireComeToState(context, path,
                                "1.   ");

                            // 1.    []
                            return new StateMachineConfig(
                                "1.   ");
                        }
                        // 1.   -> 1.   
                        // e1[o2.x1 && !o2.x4]/o2.z1,o1.z10
                        fireTransitionCandidate(context, path,
                            "1.   ", event,
                            "1.   #1.   "
                                + "#e1#o2.x1 && !o2.x4");
                        if (!isInputActionCalculated(
                            calculatedInputActions, _o2_x4)) {
                            fireBeforeInputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && !o2.x4",
                                "o2.x4");
                            o2_x4 = o2.x4(context);
                            fireAfterInputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && !o2.x4",
                                "o2.x4", new Boolean(o2_x4));
                        }
                        if (o2_x1 && !o2_x4) {
                            fireTransitionFound(context, path,
                                "1.   ", event,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && !o2.x4");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && !o2.x4",
                                "o2.z1");
                            o2.z1(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && !o2.x4",
                                "o2.z1");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && !o2.x4",
                                "o1.z10");
                            o1.z10(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && !o2.x4",
                                "o1.z10");
                            fireComeToState(context, path,
                                "1.   ");

                            // 1.    []
                            return new StateMachineConfig(
                                "1.   ");
                        }
                        // 1.   ->1.   
                        // e1[o2.x1 && o2.x4]/o2.z2,o1.z8
                        fireTransitionCandidate(context, path,
                            "1.   ", event,
                            "1.   #" +
                                "1.   #e1#o2.x1 && o2.x4");
                        if (o2_x1 && o2_x4) {
                            fireTransitionFound(context, path,
                                "1.   ", event,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && o2.x4");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && o2.x4",
                                "o2.z2");
                            o2.z2(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && o2.x4",
                                "o2.z2");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && o2.x4",
                                "o1.z8");
                            o1.z8(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #" +
                                    "e1#o2.x1 && o2.x4",
                                "o1.z8");
                            fireComeToState(context, path,
                                "1.   ");

                            // 1.    []
                            return new StateMachineConfig(
                                "1.   ");
                        }

                        // transition not found
                        return config;
                    case e3:
                        // 1.   ->
                        // 1.    e3[!o2.x3]/
                        fireTransitionCandidate(context, path,
                            "1.   ", event,
                            "1.   #" +
                                "1.   #e3#!o2.x3");
                        if (!isInputActionCalculated(
                            calculatedInputActions, _o2_x3)) {
                            fireBeforeInputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e3#!o2.x3",
                                "o2.x3");
                            o2_x3 = o2.x3(context);
                            fireAfterInputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e3#!o2.x3",
                                "o2.x3", new Boolean(o2_x3));
                        }
                        if (!o2_x3) {
                            fireTransitionFound(context, path,
                                "1.   ", event,
                                "1.   #" +
                                    "1.   #e3#!o2.x3");
                            fireComeToState(context, path,
                                "1.   ");

                            // 1.    []
                            return new StateMachineConfig(
                                "1.   ");
                        }
                        // 1.   ->1.   
                        // e3[o2.x3]/o2.z4,o1.z10,o1.z4
                        fireTransitionCandidate(context, path,
                            "1.   ", event,
                            "1.   #" +
                                "1.   #e3#o2.x3");
                        if (o2_x3) {
                            fireTransitionFound(context, path,
                                "1.   ", event,
                                "1.   #" +
                                    "1.   #e3#o2.x3");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e3#o2.x3",
                                "o2.z4");
                            o2.z4(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e3#o2.x3",
                                "o2.z4");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e3#o2.x3",
                                "o1.z10");
                            o1.z10(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e3#o2.x3",
                                "o1.z10");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e3#o2.x3",
                                "o1.z4");
                            o1.z4(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.   #" +
                                    "1.   #e3#o2.x3",
                                "o1.z4");
                            fireComeToState(context, path,
                                "1.   ");

                            // 1.    []
                            return new StateMachineConfig(
                                "1.   ");
                        }

                        // transition not found
                        return config;
                    case e6:
                        // 1.   ->
                        // 2.   e6[true]/o1.z13
                        fireTransitionCandidate(context, path,
                            "1.   ", event,
                            "1.   #" +
                                "2.  #e6#true");
                        fireTransitionFound(context, path,
                            "1.   ", event,
                            "1.   #" +
                                "2.  #e6#true");
                        fireBeforeOutputActionExecution(context, path,
                            "1.   #" +
                                "2.  #e6#true",
                            "o1.z13");
                        o1.z13(context);
                        fireAfterOutputActionExecution(context, path,
                            "1.   #" +
                                "2.  #e6#true",
                            "o1.z13");
                        fireComeToState(context, path,
                            "2.  ");

                        // 2.   []
                        return new StateMachineConfig(
                            "2.  ");
                    default:

                        // transition not found
                        return config;
                    }
                default:
                    throw new EventProcessorException(
                        "Incorrect stable state [" + config.getActiveState()
                            + "] in state machine [A1]");
                }
            }
        }

        //   
        //o2.x3
        private static final int _o2_x3 = 0;
        //o2.x1
        private static final int _o2_x1 = 1;
        //o2.x4
        private static final int _o2_x4 = 2;
    }

    /**
     * ,   A2.
     */
    private class A2EventProcessor {
        //    (  Top 
        //   )
        private static final int Top = 1;
        private static final int _1___ = 2;
        private static final int _4__ = 3;
        private static final int _5___ = 4;
        private static final int _6___ = 5;
        private static final int _0___ = 6;
        private static final int _3___ = 7;
        private static final int _2___ = 8;

        //  ,     
        //   
        private int decodeState(String state) {
            if ("Top".equals(state)) {
                return Top;
            } else if ("1.  ".equals(state)) {
                return _1___;
            } else if ("4. ".equals(state)) {
                return _4__;
            } else if ("5.  ".equals(state)) {
                return _5___;
            } else if ("6.  ".equals(state)) {
                return _6___;
            } else if ("0.  ".equals(state)) {
                return _0___;
            } else if ("3.  ".equals(state)) {
                return _3___;
            } else if ("2.  ".equals(state)) {
                return _2___;
            }
            return -1;
        }

        //  
        private static final int e4 = 1;
        private static final int e2 = 2;
        private static final int e5 = 3;

        //  ,     
        //   
        private int decodeEvent(String event) {
            if ("e4".equals(event)) {
                return e4;
            } else if ("e2".equals(event)) {
                return e2;
            } else if ("e5".equals(event)) {
                return e5;
            }
            return -1;
        }

        //    A2
        private co.DrinkMakingDevice o3;
        private co.ColdStore o5;
        private co.PourOutDevice o4;
        private co.Display o1;
        private co.CoinMechanism o2;

        //     A2
        private void init(ControlledObjectsMap controlledObjectsMap) {
            o3 = (co.DrinkMakingDevice)
                controlledObjectsMap.getControlledObject("o3");
            o5 = (co.ColdStore)
                controlledObjectsMap.getControlledObject("o5");
            o4 = (co.PourOutDevice)
                controlledObjectsMap.getControlledObject("o4");
            o1 = (co.Display)
                controlledObjectsMap.getControlledObject("o1");
            o2 = (co.CoinMechanism)
                controlledObjectsMap.getControlledObject("o2");
        }

        //    :   
        // ,    
        private StateMachineConfig process(Event event,
            StateMachineContext context, StateMachinePath path,
            StateMachineConfig config) throws Exception {
            config = lookForTransition(event, context, path, config);
            config = transiteToStableState(context, path, config);

            // execute included state machines
            executeSubmachines(event, context, path, config);
            return config;
        }

        //    
        private void executeSubmachines(Event event,
            StateMachineContext context, StateMachinePath path,
            StateMachineConfig config) throws Exception {
            int state = decodeState(config.getActiveState());
            while (true) {
                switch (state) {
                case _1___:
                    return;
                case _4__:
                    return;
                case _5___:
                    state = _4__;
                    break;
                case _6___:
                    state = _4__;
                    break;
                case _0___:
                    return;
                case _3___:
                    return;
                case _2___:
                    return;
                default:
                    throw new EventProcessorException(
                        "State with name [" + config.getActiveState()
                            + "] is unknown for state machine [A2]");
                }
            }
        }

        //    A2
        private StateMachineConfig transiteToStableState(
            StateMachineContext context, StateMachinePath path,
            StateMachineConfig config) throws Exception {
            int s = decodeState(config.getActiveState());
            Event event;
            switch (s) {
            case Top:
                fireComeToState(context, path,
                    "0.  ");

                // 0.  ->
                // 1.   [true]/o5.z0,o4.z0,o3.z0
                event = Event.NO_EVENT;
                fireTransitionFound(context, path,
                    "0.  ", event,
                    "0.  #1.  ##true");
                fireBeforeOutputActionExecution(context, path,
                    "0.  #1.  ##true",
                    "o5.z0");
                o5.z0(context);
                fireAfterOutputActionExecution(context, path,
                    "0.  #1.  ##true",
                    "o5.z0");
                fireBeforeOutputActionExecution(context, path,
                    "0.  #1.  ##true",
                    "o4.z0");
                o4.z0(context);
                fireAfterOutputActionExecution(context, path,
                    "0.  #1.  ##true",
                    "o4.z0");
                fireBeforeOutputActionExecution(context, path,
                    "0.  #1.  ##true",
                    "o3.z0");
                o3.z0(context);
                fireAfterOutputActionExecution(context, path,
                    "0.  #1.  ##true",
                    "o3.z0");
                fireComeToState(context, path, "1.  ");

                // 1.   []
                return new StateMachineConfig("1.  ");
            case _4__:
                fireCompositeTargetState(context, path,
                    "4. ");
                fireComeToState(context, path,
                    "5.  ");

                // 5.  ->6.   [true]/
                event = Event.NO_EVENT;
                fireTransitionFound(context, path,
                    "5.  ", event,
                    "5.  #6.  ##true");
                fireComeToState(context, path,
                    "6.  ");

                // 6.   []
                return new StateMachineConfig(
                    "6.  ");
            }
            return config;
        }

        //    (   
        //   ).   ,
        //   ,
        //    .
        private StateMachineConfig lookForTransition(Event event,
            StateMachineContext context, StateMachinePath path,
            StateMachineConfig config) throws Exception {
            boolean o2_x3 = false,
                o4_x1 = false,
                o3_x2 = false,
                o3_x1 = false,
                o2_x2 = false;
            BitSet calculatedInputActions = new BitSet(5);
            int s = decodeState(config.getActiveState());
            int e = decodeEvent(event.getName());
            while (true) {
                switch (s) {
                case _1___:
                    switch (e) {
                    case e2:
                        // 1.  ->2.  
                        // e2[o2.x3 && o2.x2 && !o3.x1]/
                        fireTransitionCandidate(context, path,
                            "1.  ", event,
                            "1.  #2.  #" +
                                "e2#o2.x3 && o2.x2 && !o3.x1");
                        if (!isInputActionCalculated(
                            calculatedInputActions, _o2_x3)) {
                            fireBeforeInputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o2.x3");
                            o2_x3 = o2.x3(context);
                            fireAfterInputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o2.x3", new Boolean(o2_x3));
                        }
                        if (!isInputActionCalculated(
                            calculatedInputActions, _o3_x1)) {
                            fireBeforeInputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.x1");
                            o3_x1 = o3.x1(context);
                            fireAfterInputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.x1", new Boolean(o3_x1));
                        }
                        if (!isInputActionCalculated(
                            calculatedInputActions, _o2_x2)) {
                            fireBeforeInputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o2.x2");
                            o2_x2 = o2.x2(context);
                            fireAfterInputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o2.x2", new Boolean(o2_x2));
                        }
                        if (o2_x3 && o2_x2 && !o3_x1) {
                            fireTransitionFound(context, path,
                                "1.  ", event,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1");
                            fireComeToState(context, path,
                                "2.  ");

                            // 2.   [o1.z5, o2.z3,
                            // o1.z10, o3.z1, o3.z4, o3.z2, o3.z5, o3.z3]
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o1.z5");
                            o1.z5(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o1.z5");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o2.z3");
                            o2.z3(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o2.z3");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o1.z10");
                            o1.z10(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o1.z10");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.z1");
                            o3.z1(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.z1");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.z4");
                            o3.z4(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.z4");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.z2");
                            o3.z2(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.z2");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.z5");
                            o3.z5(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.z5");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.z3");
                            o3.z3(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.  #2.  #" +
                                    "e2#o2.x3 && o2.x2 && !o3.x1",
                                "o3.z3");
                            return new StateMachineConfig(
                                "2.  ");
                        }
                        // 1.  ->
                        // 1.   e2[o2.x3 && o3.x1]/o1.z7
                        fireTransitionCandidate(context, path,
                            "1.  ", event,
                            "1.  #1.  #" +
                                "e2#o2.x3 && o3.x1");
                        if (o2_x3 && o3_x1) {
                            fireTransitionFound(context, path,
                                "1.  ", event,
                                "1.  #1.  #" +
                                    "e2#o2.x3 && o3.x1");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.  #1.  #" +
                                    "e2#o2.x3 && o3.x1",
                                "o1.z7");
                            o1.z7(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.  #1.  #" +
                                    "e2#o2.x3 && o3.x1",
                                "o1.z7");
                            fireComeToState(context, path,
                                "1.  ");

                            // 1.   []
                            return new StateMachineConfig(
                                "1.  ");
                        }
                        // 1.  ->1.   e2[!o2.x3]/
                        fireTransitionCandidate(context, path,
                            "1.  ", event,
                            "1.  #1.  #e2#!o2.x3");
                        if (!o2_x3) {
                            fireTransitionFound(context, path,
                                "1.  ", event,
                                "1.  #1.  #e2#!o2.x3");
                            fireComeToState(context, path,
                                "1.  ");

                            // 1.   []
                            return new StateMachineConfig(
                                "1.  ");
                        }
                        // 1.  ->1.  
                        // e2[o2.x3 && !o2.x2 && !o3.x1]/o1.z3
                        fireTransitionCandidate(context, path,
                            "1.  ", event,
                            "1.  #1.  #" +
                                "e2#o2.x3 && !o2.x2 && !o3.x1");
                        if (o2_x3 && !o2_x2 && !o3_x1) {
                            fireTransitionFound(context, path,
                                "1.  ", event,
                                "1.  #1.  #" +
                                    "e2#o2.x3 && !o2.x2 && !o3.x1");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "1.  #1.  #" +
                                    "e2#o2.x3 && !o2.x2 && !o3.x1",
                                "o1.z3");
                            o1.z3(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "1.  #1.  #" +
                                    "e2#o2.x3 && !o2.x2 && !o3.x1",
                                "o1.z3");
                            fireComeToState(context, path,
                                "1.  ");

                            // 1.   []
                            return new StateMachineConfig(
                                "1.  ");
                        }

                        // transition not found
                        return config;
                    default:

                        // transition not found
                        return config;
                    }
                case _4__:
                    fireTransitionsOfSuperstate(context, path,
                        "4. ", event);
                    switch (e) {
                    default:

                        // transition not found
                        return config;
                    }
                case _3___:
                    switch (e) {
                    case e4:

                        // 3.  ->
                        // 4.  e4[o3.x2 || o4.x1]/
                        fireTransitionCandidate(context, path,
                            "3.  ", event,
                            "3.  #4. #" +
                                "e4#o3.x2 || o4.x1");
                        if (!isInputActionCalculated(
                            calculatedInputActions, _o4_x1)) {
                            fireBeforeInputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o4.x1");
                            o4_x1 = o4.x1(context);
                            fireAfterInputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o4.x1", new Boolean(o4_x1));
                        }
                        if (!isInputActionCalculated(
                            calculatedInputActions, _o3_x2)) {
                            fireBeforeInputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o3.x2");
                            o3_x2 = o3.x2(context);
                            fireAfterInputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o3.x2", new Boolean(o3_x2));
                        }
                        if (o3_x2 || o4_x1) {
                            fireTransitionFound(context, path,
                                "3.  ", event,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1");
                            fireComeToState(context, path,
                                "4. ");

                            // 4.  [o2.z4, o1.z10, o1.z9,
                            // o3.z6, o4.z4, o5.z1]
                            fireBeforeOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o2.z4");
                            o2.z4(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o2.z4");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o1.z10");
                            o1.z10(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o1.z10");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o1.z9");
                            o1.z9(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o1.z9");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o3.z6");
                            o3.z6(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o3.z6");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o4.z4");
                            o4.z4(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o4.z4");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o5.z1");
                            o5.z1(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "3.  #4. #" +
                                    "e4#o3.x2 || o4.x1",
                                "o5.z1");
                            return new StateMachineConfig(
                                "4. ");
                        }
                        // 3.  ->
                        // 1.   e4[!o3.x2 && !o4.x1]/
                        fireTransitionCandidate(context, path,
                            "3.  ", event,
                            "3.  #1.  #" +
                                "e4#!o3.x2 && !o4.x1");
                        if (!o3_x2 && !o4_x1) {
                            fireTransitionFound(context, path,
                                "3.  ", event,
                                "3.  #1.  #" +
                                    "e4#!o3.x2 && !o4.x1");
                            fireComeToState(context, path,
                                "1.  ");

                            // 1.   []
                            return new StateMachineConfig(
                                "1.  ");
                        }

                        // transition not found
                        return config;
                    default:

                        // transition not found
                        return config;
                    }
                case _2___:
                    switch (e) {
                    case e5:

                        // 2.  ->
                        // 3.   e5[!o3.x1]/
                        fireTransitionCandidate(context, path,
                            "2.  ", event,
                            "2.  #" +
                                "3.  #e5#!o3.x1");
                        if (!isInputActionCalculated(
                            calculatedInputActions, _o3_x1)) {
                            fireBeforeInputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#!o3.x1",
                                "o3.x1");
                            o3_x1 = o3.x1(context);
                            fireAfterInputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#!o3.x1",
                                "o3.x1", new Boolean(o3_x1));
                        }
                        if (!o3_x1) {
                            fireTransitionFound(context, path,
                                "2.  ", event,
                                "2.  #" +
                                    "3.  #e5#!o3.x1");
                            fireComeToState(context, path,
                                "3.  ");

                            // 3.   [o4.z1, o4.z3, o4.z2, o1.z6]
                            fireBeforeOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#!o3.x1",
                                "o4.z1");
                            o4.z1(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#!o3.x1",
                                "o4.z1");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#!o3.x1",
                                "o4.z3");
                            o4.z3(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#!o3.x1",
                                "o4.z3");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#!o3.x1",
                                "o4.z2");
                            o4.z2(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#!o3.x1",
                                "o4.z2");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#!o3.x1",
                                "o1.z6");
                            o1.z6(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#!o3.x1",
                                "o1.z6");
                            return new StateMachineConfig(
                                "3.  ");
                        }
                        // 2.  ->
                        // 3.   e5[o3.x1]/o1.z12
                        fireTransitionCandidate(context, path,
                            "2.  ", event,
                            "2.  #" +
                                "3.  #e5#o3.x1");
                        if (o3_x1) {
                            fireTransitionFound(context, path,
                                "2.  ", event,
                                "2.  #" +
                                    "3.  #e5#o3.x1");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#o3.x1",
                                "o1.z12");
                            o1.z12(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#o3.x1",
                                "o1.z12");
                            fireComeToState(context, path,
                                "3.  ");

                            // 3.   [o4.z1, o4.z3, o4.z2, o1.z6]
                            fireBeforeOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#o3.x1",
                                "o4.z1");
                            o4.z1(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#o3.x1",
                                "o4.z1");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#o3.x1",
                                "o4.z3");
                            o4.z3(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#o3.x1",
                                "o4.z3");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#o3.x1",
                                "o4.z2");
                            o4.z2(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#o3.x1",
                                "o4.z2");
                            fireBeforeOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#o3.x1",
                                "o1.z6");
                            o1.z6(context);
                            fireAfterOutputActionExecution(context,
                                path,
                                "2.  #" +
                                    "3.  #e5#o3.x1",
                                "o1.z6");
                            return new StateMachineConfig(
                                "3.  ");
                        }

                        // transition not found
                        return config;
                    default:

                        // transition not found
                        return config;
                    }
                default:
                    throw new EventProcessorException(
                        "Incorrect stable state [" + config.getActiveState()
                            + "] in state machine [A2]");
                }
            }
        }

        //   
        //o2.x3
        private static final int _o2_x3 = 0;
        //o4.x1
        private static final int _o4_x1 = 1;
        //o3.x2
        private static final int _o3_x2 = 2;
        //o3.x1
        private static final int _o3_x1 = 3;
        //o2.x2
        private static final int _o2_x2 = 4;
    }

    //     ,    
    //   
    private static boolean isInputActionCalculated(
        BitSet calculatedInputActions, int k) {
        boolean b = calculatedInputActions.get(k);
        if (!b) {
            calculatedInputActions.set(k);
        }
        return b;
    }
}