package co.ui;

import com.evelopers.unimod.core.stateworks.Event;
import com.evelopers.unimod.runtime.ModelEngine;
import com.evelopers.unimod.runtime.context.Parameter;
import com.evelopers.unimod.runtime.context.StateMachineContext;
import com.evelopers.unimod.runtime.context.StateMachineContextImpl;
import common.Parameters;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.border.MatteBorder;
import javax.swing.text.html.HTMLEditorKit;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

/**
 *       
 *      .
 */
public class ControlPaneView implements VendingMachine,
                                        ActionListener {
    private JFrame frame;
    private JLabel canPicture;
    private JLabel payBackPicture;
    private Image bgimage = null;
    private JRadioButton[] radio;
    private JLabel lblPrepayment;
    private JEditorPane display;
    private JButton moneyBack;

    private int returnedCoinsCount;

    private static final Color NICE_DARK_GRAY = new Color(75, 75, 75);
    private static final Color NICE_GRAY = new Color(204, 204, 204);
    private static final Color NICE_WHITE = new Color(238, 238, 238);

    private ModelEngine engine = null;
    private static VendingMachine vm = new ControlPaneView();

    /**
     *    .
     */
    private ControlPaneView() {
        frame = new JFrame();
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createUI();
            }
        });
    }

    /**
     *  ,   .
     *
     * @return ,   .
     */
    public Container getContentPane() {
        return frame.getContentPane();
    }

    /**
     *  "" ,    
     *      .
     *
     * @param engine    .
     */
    public void setEngine(ModelEngine engine) {
        this.engine = engine;
    }

    /**
     *      .
     *
     * @return    .
     */
    public static VendingMachine getVm() {
        return vm;
    }

    /**
     *       .
     *
     * @param event .
     */
    public void fireEvent(String event) {
        engine.getEventManager().handle(
                new Event(event), StateMachineContextImpl.create());
    }

    /**
     *       .
     *
     * @param event  .
     * @param pName   .
     * @param pValue  .
     */
    public void fireEvent(String event, String pName, Object pValue) {
        StateMachineContext context = StateMachineContextImpl.create();
        context.getApplicationContext().setParameter(pName, pValue);
        engine.getEventManager().handle(new Event(event,
                new Parameter[]{new Parameter(pName, pValue)}),
                context);
    }

    /**
     *     .
     *
     * @param a  ,  ;
     *           .
     */
    public void setVisible(final boolean a) {
        if (Parameters.runningMode == Parameters.RUN_MODE_FRAME) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    frame.setVisible(a);
                }
            });
        }
    }

    /**
     *   ,   .
     */
    public void dispose() {
        frame.dispose();
    }

    /**
     *    .
     */
    private void createUI() {
        final int WIDTH = 430;
        final int HEIGHT = 640;
        final int COIN_HEIGHT = 70;
        final int COIN_WIDTH = 1 + WIDTH / Parameters.NUM_COINS;

        frame.setTitle(Parameters.getMessage(
            "ControlPaneView.Caption"));
        frame.setResizable(false);
        frame.setDefaultCloseOperation(
            WindowConstants.DO_NOTHING_ON_CLOSE);
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                fireEvent(ep.ControlPane.E6);
            }
        });
        frame.setSize(WIDTH, HEIGHT);
        Dimension ss = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setLocation((ss.width - WIDTH) >> 1,
            (ss.height - HEIGHT) >> 1);
        Container contentPane = frame.getContentPane();
        contentPane.setLayout(null);

        for (int i = 0; i < Parameters.NUM_COINS; i++) {
            ImageIcon picCoin = Parameters.getImageIcon(
                "coin-picture-" + i);
            JPanel coinDragPanel = new JPanel();
            coinDragPanel.setBounds(i * COIN_WIDTH, 545,
                COIN_WIDTH, COIN_HEIGHT);
            coinDragPanel.setBackground(ControlPaneView.NICE_WHITE);
            coinDragPanel.add((new MoneyDragProvider(
                    Integer.parseInt(Parameters.getMessage(
                        "ControlPaneView.Coins.FaceValue." + i)),
                    picCoin,
                    Parameters.getMessage(
                        "ControlPaneView.Coins.ToolTip." + i)))
                .getTransferable());
            contentPane.add(coinDragPanel);
        }

        bgimage = Parameters.getImageIcon(
            "background-picture").getImage();
        JPanel content = new JPanel(null) {
			private static final long serialVersionUID
                = -1933987628741217471L;

			protected void paintComponent(Graphics g) {
                super.paintComponent(g);

                if (bgimage != null) {
                    int imwidth = bgimage.getWidth(null);
                    int imheight = bgimage.getHeight(null);
                    if ((imwidth > 0) && (imheight > 0)) {
                        g.drawImage(bgimage, 0, 0, null);
                    }
                }
            }
        };

        content.setBounds(0, 0, WIDTH, HEIGHT - COIN_HEIGHT);
        contentPane.add(content);

        final int POS_X = 216;
        final int POS_Y = 78;
        final int PANEL_WIDTH = 192;
        final int PANEL_HEIGHT = 50;
        final int DRINK_PIC_WIDTH = 100;

        JComponent coinDropPanel = (new MoneyDropProvider())
            .createDropPanel();
        coinDropPanel.setBounds(POS_X, POS_Y, 14, PANEL_HEIGHT);
        content.add(coinDropPanel);

        JLabel coinHinter = new JLabel(Parameters.getImageIcon(
            "drop-coin-here-hint"));
        coinHinter.setToolTipText(Parameters.getMessage(
            "CoinHinter.ToolTipText"));
        coinHinter.setBackground(NICE_WHITE);
        coinHinter.setOpaque(true);
        coinHinter.setBorder(
                new MatteBorder(2, 0, 2, 2, NICE_GRAY));
        coinHinter.setBounds(POS_X + 14, POS_Y, 75, PANEL_HEIGHT);
        content.add(coinHinter);

        lblPrepayment = new JLabel(Parameters.getMessage(
            "Display.Sum", 0));
        lblPrepayment.setHorizontalAlignment(SwingConstants.CENTER);
        lblPrepayment.setBackground(NICE_WHITE);
        lblPrepayment.setOpaque(true);
        lblPrepayment.setBorder(
                new MatteBorder(2, 0, 2, 2, NICE_GRAY));
        lblPrepayment.setBounds(POS_X + 89, POS_Y, 103, PANEL_HEIGHT);
        content.add(lblPrepayment);

        JLabel box = new JLabel();
        box.setBorder(new MatteBorder(0, 2, 2, 2, NICE_GRAY));
        box.setBounds(POS_X, POS_Y + PANEL_HEIGHT,
            PANEL_WIDTH, PANEL_HEIGHT);
        content.add(box);

        display = new JEditorPane();
        display.setBackground(NICE_DARK_GRAY);
        display.setEditable(false);
        display.setEditorKit(new HTMLEditorKit());
        display.setText(Parameters.getMessage("Display.Message",
            Parameters.getMessage("Message.BuyDrink")));
        display.setMargin(new Insets(0, 2, 0, 0));
        display.setBounds(POS_X + 2, POS_Y + PANEL_HEIGHT,
            PANEL_WIDTH - 4, PANEL_HEIGHT - 2);
        content.add(display);

        JLabel[] lblPrices = new JLabel[Parameters.NUM_DRINKS];
        radio = new JRadioButton[Parameters.NUM_DRINKS];
        for (int i = 0; i < Parameters.NUM_DRINKS; i++) {
            int curr_y = POS_Y + (PANEL_HEIGHT + 2) * i
                + 2 * PANEL_HEIGHT;
            JLabel drinkPic = new JLabel(Parameters.getImageIcon(
                "drink-picture-" + i));
            drinkPic.setToolTipText(Parameters.getMessage(
                "ControlPaneView.Drinks." + i));
            drinkPic.setBounds(POS_X, curr_y,
                DRINK_PIC_WIDTH, PANEL_HEIGHT);
            content.add(drinkPic);

            radio[i] = new JRadioButton(Parameters.getImageIcon(
                "indicator-initial"));
            radio[i].setActionCommand(Integer.toString(i));
            radio[i].setBackground(NICE_WHITE);
            radio[i].addActionListener(this);
            radio[i].setPressedIcon(Parameters.getImageIcon(
                "indicator-pressed"));
            radio[i].setMargin(new Insets(0, 10, 0, 0));
            radio[i].setBounds(POS_X + DRINK_PIC_WIDTH, curr_y,
                40, PANEL_HEIGHT);
            content.add(radio[i]);

            lblPrices[i] = new JLabel(Parameters.getMessage(
                "PriceLabel.Caption",
                    Parameters.getMessage(
                        "ControlPaneView.Drinks.Price." + i)));
            lblPrices[i].setBounds(POS_X + DRINK_PIC_WIDTH + 40,
                curr_y, PANEL_WIDTH - DRINK_PIC_WIDTH - 40,
                PANEL_HEIGHT);
            lblPrices[i].setBackground(NICE_WHITE);
            lblPrices[i].setOpaque(true);
            content.add(lblPrices[i]);

            if (i < Parameters.NUM_DRINKS - 1) {
                JLabel splitter = new JLabel();
                splitter.setBackground(NICE_GRAY);
                splitter.setOpaque(true);
                splitter.setBounds(POS_X, curr_y + PANEL_HEIGHT,
                    PANEL_WIDTH, 2);
                content.add(splitter);
            }
        }

        int curr_y = POS_Y + PANEL_HEIGHT
            * (2 + Parameters.NUM_DRINKS) + 8;
        canPicture = new JLabel(Parameters.getImageIcon(
            "drink-box-no-drink"));
        canPicture.setBounds(POS_X, curr_y, 50, 50);
        content.add(canPicture);

        payBackPicture = new JLabel(Parameters.getImageIcon(
            "money-back-0"));
        payBackPicture.setBounds(POS_X + 50, curr_y, 71, 50);
        content.add(payBackPicture);

        moneyBack = new JButton(Parameters.getMessage(
            "MoneyBackButton.Caption"));
        moneyBack.addActionListener(this);
        moneyBack.setBounds(POS_X + 121, curr_y, 71, 50);
        content.add(moneyBack);
    }

    /**
     *       .
     *
     * @param e .
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == moneyBack) {
            fireEvent(ep.ControlPane.E3);
            return;
        }
        JRadioButton radioButton = ((JRadioButton) e.getSource());
        radioButton.setSelected(false);
        fireEvent(ep.ControlPane.E2, Parameters.REQUESTED_DRINK,
                new Integer(radioButton.getActionCommand()));
    }

    /**
     *  .
     *
     * @param coins  .
     */
    public void drawCoins(int coins) {
        int d = (coins > 6 ? 6 : coins);
        if (coins != 0) {
            d += returnedCoinsCount;
            if (d > 6) {
                d = 6;
            }
        }
        final int c = d;
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                returnedCoinsCount = c;
                payBackPicture.setIcon(Parameters.getImageIcon(
                    "money-back-" + c));
            }
        });
    }

    /**
     *    .
     *
     * @param message ,   .
     */
    public void drawMessage(final String message) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                display.setText(Parameters.getMessage(
                    "Display.Message", message));
            }
        });
    }

    /**
     *     .
     *
     * @param showCan  ,   ;
     *                  .
     */
    public void drawCan(final boolean showCan) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (showCan) {
                    canPicture.setIcon(Parameters.getImageIcon(
                        "drink-box-drink"));
                } else {
                    canPicture.setIcon(Parameters.getImageIcon(
                        "drink-box-no-drink"));
                }
            }
        });
    }

    /**
     *   .
     *
     * @param prepayment .
     */
    public void drawPrepayment(final int prepayment) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                lblPrepayment.setText(Parameters.getMessage(
                    "Display.Sum", prepayment));
            }
        });
    }

    /**
     *       .
     *
     * @param indicatorID  .
     */
    public void drawTurnedOffIndicator(final int indicatorID) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                radio[indicatorID].setIcon(Parameters.getImageIcon(
                    "indicator-off"));
            }
        });
    }

    /**
     *  ,   
     *     .
     */
    private static class MoneyDragProvider implements
                                           DragGestureListener {
        private final JComponent transferable;
        private int value = 0;

        /**
         *  ""  .
         *
         * @param v        .
         * @param image   .
         * @param toolTip   .
         */
        public MoneyDragProvider(int v, ImageIcon image,
            String toolTip) {
            value = v;
            transferable = new JLabel(image);
            transferable.setToolTipText(toolTip);

            DragSource dragSource = DragSource.getDefaultDragSource();
            dragSource.createDefaultDragGestureRecognizer(
                    transferable,
                    DnDConstants.ACTION_COPY_OR_MOVE,
                    this);
        }

        /**
         *  ""  .
         *
         * @return ""  .
         */
        public JComponent getTransferable() {
            return transferable;
        }

        /**
         *  "":  .
         *
         * @param e .
         */
        public void dragGestureRecognized(DragGestureEvent e) {
            e.startDrag(Parameters.COIN_CURSOR,
                    new StringSelection(Integer.toString(value)),
                    null);
        }
    }

    /**
     *  .
     */
    private class MoneyDropProvider extends DropTargetAdapter {
        /**
         *  ,      
         *    drag-and-drop.
         *
         * @return ,      .
         */
        public JPanel createDropPanel() {
            JPanel panel = new JPanel();
            panel.setToolTipText(Parameters.getMessage(
                "MoneyDropProvider.ToolTipText"));
            new DropTarget(panel,
                    DnDConstants.ACTION_COPY_OR_MOVE,
                    this);
            panel.setBackground(NICE_DARK_GRAY);
            panel.setBorder(new LineBorder(NICE_GRAY, 2));
            return panel;
        }

        /**
         *    :  
         *    drag-and-drop    .
         *
         * @param e .
         */
        public void drop(DropTargetDropEvent e) {
            try {
                DataFlavor stringFlavor = DataFlavor.stringFlavor;
                Transferable tr = e.getTransferable();
                if (e.isDataFlavorSupported(stringFlavor)) {
                    String data = (String) tr.getTransferData(
                        stringFlavor);
                    e.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
                    e.dropComplete(true);
                    fireEvent(ep.ControlPane.E1, Parameters.COIN_TYPE,
                            new Integer(data));
                } else {
                    e.rejectDrop();
                }
            } catch (IOException ex) {
                ex.printStackTrace();
            } catch (UnsupportedFlavorException ufe) {
                ufe.printStackTrace();
            }
        }
    }
}